==> mlr3proba_0.4.2.tar.gz: mlr3proba/R/surv_return.R <==
#' @title Get Survival Predict Types
#' @description Internal helper function to easily return the correct survival predict types and to
#' automatically coerce a predicted survival probability matrix to a
#' [distr6::VectorDistribution] of [distr6::WeightedDiscrete] distributions.
#' @param times (`numeric()`) \cr Vector of survival times.
#' @param surv (`matrix()`)\cr Matrix of predicted survival probabilities, rows are observations,
#' columns are times. Number of columns should be equal to length of `times`.
#' @param crank (`numeric()`)\cr Relative risk/continuous ranking. Higher value is associated
#' with higher risk. If `NULL` then either set as `lp` if available or as the estimated
#' survival expectation, computed by `colSums(surv)`.
#' @param lp (`numeric()`)\cr Predicted linear predictor, used to impute `crank` if `NULL`.
#' @param response (`numeric()`)\cr Predicted survival time, passed through function without
#' modification.
#' @details
#' To avoid complications caused by degenerative distributions, if `0` is not in `times`, then
#' this is added as the first time-point and a column of `1`s is bound to the left of `surv`.
#' Additionally if the final column of `surv` is not all `1`s, then an additional time is added
#' as `max(times) + 1e-3` and a column of `1`s is bound to the right of `surv`.
#' @export
.surv_return = function(times = NULL, surv = NULL, crank = NULL, lp = NULL, response = NULL) {

  out = list()

  if ((!is.null(times) && is.null(surv)) || (is.null(times) && !is.null(surv))) {
    stop("Either both `times` and `surv` should be `NULL` or neither should be.")
  } else if (is.null(times) && is.null(surv)) {
    distr = NULL
  } else {
    # assume surv rows = observations and cols = times
    assert(length(times) == ncol(surv))

    # add predictions at time = 0 if not already present
    if (0 %nin% times) {
      times = c(0, times)
      surv = cbind(1, surv)
    }

    # add S(t) = 1 if not already present
    if (!all(surv[, ncol(surv)] == numeric(nrow(surv)))) {
      times = c(times, max(times) + 1e-3)
      surv = cbind(surv, 0)
    }
    colnames(surv) = times
    out$distr = surv
  }

  if (is.null(crank)) {
    if (!is.null(response)) {
      # low survival time = high risk
      # high crank = high risk
      crank = -response
    } else if (!is.null(lp)) {
      # assumes PH-type lp where high value = high risk
      crank = lp
    } else if (!is.null(surv)) {
      # negative mean survival time
      crank = -apply(1 - surv, 1, function(.x) sum(c(.x[1], diff(.x)) * times))
    }
  }

  out$crank = crank
  out$lp = lp
  out$response = response

  out
}

==> mlr3proba_0.4.9.tar.gz: mlr3proba/R/surv_return.R <==
#' @title Get Survival Predict Types
#' @description Internal helper function to easily return the correct survival predict types and to
#' automatically coerce a predicted survival probability matrix to a [distr6::Matdist].
#' @param times (`numeric()`) \cr Vector of survival times.
#' @param surv (`matrix()`)\cr Matrix of predicted survival probabilities, rows are observations,
#' columns are times. Number of columns should be equal to length of `times`.
#' @param crank (`numeric()`)\cr Relative risk/continuous ranking. Higher value is associated
#' with higher risk. If `NULL` then either set as `lp` if available or as the estimated
#' survival expectation, computed by `colSums(surv)`.
#' @param lp (`numeric()`)\cr Predicted linear predictor, used to impute `crank` if `NULL`.
#' @param response (`numeric()`)\cr Predicted survival time, passed through function without
#' modification.
#' @details
#' Uses [survivalmodels::surv_to_risk] to reduce survival matrices to relative
#' risks / rankings if `crank` is NULL.
#' @references
#' Sonabend, R., Bender, A., & Vollmer, S. (2021).
#' Evaluation of survival distribution predictions with discrimination
#' measures. http://arxiv.org/abs/2112.04828.
#' @export
.surv_return = function(times = NULL, surv = NULL, crank = NULL, lp = NULL, response = NULL) {

  if (!is.null(surv)) {
    times <- times %||% colnames(surv)
    assert(length(times) == ncol(surv))
    colnames(surv) <- times
  }

  if (is.null(crank)) {
    if (!is.null(response)) {
      # low survival time = high risk
      # high crank = high risk
      crank = -response
    } else if (!is.null(lp)) {
      # assumes PH-type lp where high value = high risk
      crank = lp
    } else if (!is.null(surv)) {
      crank = survivalmodels::surv_to_risk(surv)
    }
  }

  list(
    distr = surv,
    crank = crank,
    lp = lp,
    response = response
  )
}
